@@ -39,14 +39,15 @@
 #' records are not modified. Equal step in all sensors is required for conversion from Raw-format to Agg-format, otherwise
 #' period must be specified.
 #' 
-#' When fun and period are specified, microclimatic records are aggregated based on function into new period.
+#' When fun and period are specified, microclimatic records are aggregated based on a selected function into a specified period.
+#' The name of the aggregated variable will contain also the name of the function used for the aggregation (e.g. TMS_T1_mean).
 #' Aggregated time step is named after the first time step of selected period i.e. day = c(2022-12-29 00:00, 2022-12-30 00:00...);
 #' week = c(2022-12-19 00:00, 2022-12-28 00:00...); month = c(2022-11-01 00:00, 2022-12-01 00:00...);
 #' year = c(2021-01-01 00:00, 2022-01-01 00:00...).
 #' When first or last period is incomplete in original data, the incomplete part is extended with NA values to match specified period. 
-#' I.e. when you want to aggregate to monthly mean, but your time-series starts on January 15 ending December 20, 
-#' myClim will extend the time.series 
-#' to start January 1, ending December 31. Then you can decide what to do with the NAs, see parameter `min_coverage`. 
+#' For example, when you want to aggregate time-series to monthly mean, but your time-series starts on January 15 ending December 20, 
+#' myClim will extend the time-series to start on January 1 and end on December 31. 
+#' If you want to still use the data from the aggregation periods with not complete data coverage, you can adjust the parameter `min_coverage`. 
 #' 
 #' Empty sensors with no records are excluded. `mc_agg()` return NA for empty vector except from `fun=count` which returns 0.
 #' When aggregation functions are provided as vector or list e.g. c(mean, min, max), than they are all applied to all the sensors
@@ -118,13 +119,17 @@ mc_agg <- function(data, fun=NULL, period=NULL, use_utc=TRUE, percentiles=NULL,
     use_utc <- .agg_get_use_utc(data, use_utc)
     original_period <- .agg_check_steps_and_get_original_text(data, fun, period_object)
     is_raw <- .common_is_raw_format(data)
+    agg_bar <- progress::progress_bar$new(format = "agg [:bar] :current/:total localities",
+                                          total=length(data$localities))
     locality_function <- function (locality) {
         tz_offset <- if(use_utc) 0 else locality$metadata@tz_offset
         if(is_raw) {
-            return(.agg_aggregate_prep_locality(locality, fun, period, use_intervals, percentiles, min_coverage, tz_offset, custom_functions))
+            result <- .agg_aggregate_prep_locality(locality, fun, period, use_intervals, percentiles, min_coverage, tz_offset, custom_functions)
         } else {
-            return(.agg_aggregate_item(locality, fun, period, use_intervals, percentiles, min_coverage, tz_offset, original_period, custom_functions))
+            result <-.agg_aggregate_item(locality, fun, period, use_intervals, percentiles, min_coverage, tz_offset, original_period, custom_functions)
         }
+        agg_bar$tick()
+        return(result)
     }
     new_localities <- purrr::map(data$localities, locality_function)
     new_localities <- purrr::keep(new_localities, function (x) !is.null(x))
@@ -676,11 +681,17 @@ mc_agg <- function(data, fun=NULL, period=NULL, use_utc=TRUE, percentiles=NULL,
         new_sensor$values <- aggregate(new_sensor$values, by_aggregate, .x)$x
         if(.y %in% names(custom_functions)) {
             if(is.logical(new_sensor$values)) {
-                new_sensor$metadata@sensor_id <- mc_const_SENSOR_logical
+                if(!is.logical(sensor$values)) {
+                    new_sensor$metadata@sensor_id <- mc_const_SENSOR_logical
+                }
             } else if(is.integer(new_sensor$values)) {
-                new_sensor$metadata@sensor_id <- mc_const_SENSOR_integer
+                if(!is.integer(sensor$values)) {
+                    new_sensor$metadata@sensor_id <- mc_const_SENSOR_integer
+                }
             } else if(is.numeric(new_sensor$values)) {
-                new_sensor$metadata@sensor_id <- mc_const_SENSOR_real
+                if(!is.numeric(sensor$values)) {
+                    new_sensor$metadata@sensor_id <- mc_const_SENSOR_real
+                }
             } else {
                 stop(stringr::str_glue(.agg_const_MESSAGE_WRONG_CUSTOM_FUNCTION))
             }
